<?php

class E2W_Plugins_GetFiveStars_Widget
{
    protected $table;

    protected $widget_instances = 0;

    protected $hash = [];

    protected $host = [];

    protected $badge = [];

    protected $additional = [];

    protected $new = [];

    protected $advanced = [];

    protected $businessId = [];

    function __construct($table)
    {
        $this->table = $table;
        add_shortcode('getfivestars_widget', [$this, 'shortcode']);
        add_shortcode('gatherup_widget', [$this, 'shortcode']);
        add_action('wp_loaded', [$this, 'wp_loaded']);
        add_filter('widget_text', [$this, 'php_widget_text'], 99);
    }

    // execute php inside text widget
    function php_widget_text($text)
    {
        if (strpos($text, '<' . '?') !== false) {
            ob_start();
            eval('?' . '>' . $text);
            $text = ob_get_clean();
        }

        return $text;
    }

    function wp_loaded()
    {
        add_filter('the_content', [$this, 'shortcode_real'], PHP_INT_MAX);
    }

    function shortcode($attributes = [], $content = '')
    {
        global $wpdb;

        $defaults = [
            'id' => 0,
        ];
        $atts     = $this->get_defaults($attributes, $defaults);
        $id       = intval($atts['id']);

        $query  = "SELECT * FROM {$wpdb->prefix}{$this->table} WHERE id = %d";
        $widget =
            $wpdb->get_row($wpdb->prepare($query, $atts['id']), OBJECT);

        if (!is_null($widget)) {

            $this->widget_instances++;
            $unserialize = @unserialize($widget->hash);

            if (isset($unserialize['hash'])) {
                if (isset($unserialize['new'])
                    && $unserialize['new'] === true) {
                    $this->new[$id] = $unserialize['new'];
                }

                if (isset($unserialize['advanced'])
                    && $unserialize['advanced'] === true) {
                    $this->advanced[$id] = true;
                }

                if (isset($unserialize['business'])) {
                    $this->businessId[$id] = $unserialize['business'];
                }

                $this->hash[$id] = $unserialize['hash'];
                $this->host[$id] = $unserialize['host'];

                if (strpos($this->host[$id], "://") === false) {
                    $this->host[$id] = "https://" . $this->host[$id];
                }

                if (isset($unserialize['badge'])) {
                    $this->badge[$id] = true;
                } else {
                    $this->badge[$id] = false;
                }
                if (isset($unserialize['additional'])
                    and !empty($unserialize['additional'])) {
                    $this->additional[$id] = $unserialize['additional'];
                } else {
                    $this->additional[$id] = false;
                }
            } else { // backward compatibility
                $this->hash[$id] = $widget->hash;
                $this->host[$id] = 'https://getfivestars.com';
            }

            return $this->shortcode_replacement($id);
        }

        return '';
    }

    function shortcode_real($content)
    {
        $pattern = $this->shortcode_replacement_pattern();
        $result  = [];
        if (preg_match_all("/$pattern/", $content, $result)) {
            foreach ($result[1] as $_id) {
                $id      = intval($_id);
                $content = str_replace(
                    $this->shortcode_replacement($id),
                    $this->widgetHTML($id),
                    $content
                );
            }
        }

        return $content;
    }

    protected function get_defaults($attributes, $defaults = [])
    {
        $atts = shortcode_atts($defaults, $attributes);

        foreach ($atts as $key => $val) {
            $atts[$key] = trim($val);
        }

        return $atts;
    }

    protected function shortcode_replacement($id)
    {
        return '<div>e2w_getfivestars_widget_shortcode_replacement_' . $id
            . '</div>';
    }

    protected function shortcode_replacement_pattern()
    {
        $pattern = str_replace('/', '\/', $this->shortcode_replacement(0));

        return str_replace('0', '([0-9]+)', $pattern);
    }

    protected function widgetHTML($id)
    {
        $hash    = (!empty($this->hash[$id]) || $this->hash[$id] == 0);
        $isBadge = $this->badge[$id];

        ob_start();

        if (isset($this->advanced[$id]) && $this->advanced[$id] === true) {
            if ($isBadge) {
                require E2W_PLUGINS_GETFIVESTARS_WIDGET_PATH
                    . '/tpl/badge.html.php';
            } else {
                require E2W_PLUGINS_GETFIVESTARS_WIDGET_PATH
                    . '/tpl/widget.html.php';
            }
        } elseif ($hash && isset($this->new[$id]) && $this->new[$id] === true) {
            require E2W_PLUGINS_GETFIVESTARS_WIDGET_PATH
                . '/tpl/new-widget.html.php';
        } elseif ($hash) {
            $html = "";
            $url  =
                "{$this->host[$id]}/" . ($isBadge ? 'badge' : 'reviews')
                . "/{$this->hash[ $id ]}";

            if ($this->additional[$id]) {
                $url .= "/" . $this->additional[$id];
            }

            if (function_exists('curl_exec')) {
                $c = curl_init($url);
                curl_setopt($c, CURLOPT_RETURNTRANSFER, 1);
                curl_setopt($c, CURLOPT_SSL_VERIFYHOST, 0);
                curl_setopt($c, CURLOPT_CONNECTTIMEOUT, 3);
                $html = @curl_exec($c);
                @curl_close($c);
            } elseif (ini_get('allow_url_fopen')) {
                $html = @file_get_contents($url);
            }

            require E2W_PLUGINS_GETFIVESTARS_WIDGET_PATH
                . '/tpl/shortcode.html.php';
        }

        return ob_get_clean();
    }
}
